@@ -2,8 +2,6 @@ from __future__ import division
 from __future__ import print_function
 
 from math import pi
-import prodimopy.extinction
-import prodimopy.plot
 
 from matplotlib import lines
 from matplotlib import patches
@@ -14,12 +12,13 @@ import matplotlib as mpl
 import matplotlib.pyplot as plt
 import matplotlib.ticker as plticker
 import numpy as np
+import prodimopy.extinction
+import prodimopy.plot
 
 
 # has to be this way because of circular imports
 class PlotModels(object):
 
-
   def __init__(self,pdf,
                colors=None,
                styles=None,
@@ -32,7 +31,7 @@ class PlotModels(object):
       # use system default colors not fixed ones.
       # FIXME: could be done more elegant (simply set colors to None, needs changes in the plot routines)
       self.colors=[None]*20
-      #self.colors = ["b", "g", "r", "c", "m", "y", "k", "sienna", "lime", "pink", "DarkOrange", "Olive","brown"]
+      # self.colors = ["b", "g", "r", "c", "m", "y", "k", "sienna", "lime", "pink", "DarkOrange", "Olive","brown"]
     else:
       self.colors=colors
 
@@ -52,7 +51,6 @@ class PlotModels(object):
       self.fs_legend=fs_legend
     self.ncol_legend=ncol_legend
 
-
   def _sfigs(self,**kwargs):
     '''
     Scale the figure size from matplotlibrc by the factors given in the
@@ -71,7 +69,6 @@ class PlotModels(object):
     else:
       return (figsize[0],figsize[1])
 
-
   def _legend(self,ax,**kwargs):
     '''
     plots the legend, deals with multiple columns
@@ -82,7 +79,7 @@ class PlotModels(object):
     if "fs_legend" in kwargs:
       fs=kwargs["fs_legend"]
     else:
-      fs=self.fs_legend    # TODO: maybe remove fs_legend from init and just use rcparams
+      fs=self.fs_legend  # TODO: maybe remove fs_legend from init and just use rcparams
 
     if "loc_legend" in kwargs:
       loc=kwargs["loc_legend"]
@@ -97,7 +94,6 @@ class PlotModels(object):
     lw=mpl.rcParams['axes.linewidth']
     leg.get_frame().set_linewidth(lw)
 
-
   def _set_dashes(self,line):
     '''
     Utility routine to set the dashed stuff for a line.
@@ -110,7 +106,6 @@ class PlotModels(object):
     if majver<2:
       line.set_dashes((4,4))
 
-
   def _dokwargs(self,ax,**kwargs):
     '''
     Handles the passed kwargs elements (assumes that defaults are already set)
@@ -143,6 +138,27 @@ class PlotModels(object):
       if  kwargs["title"]!=None and kwargs["title"].strip()!="":
         ax.set_title(kwargs["title"])
 
+  def _initfig(self,ax=None,**kwargs):
+    '''
+    Inits Figure and Axes object.
+
+    If an Axes object is passed, it is returned together with the Figure object.
+
+    This is for a single plot (i.e. only one panel)
+
+    Returns
+    -------
+    :class:`~matplotlib.figure.Figure`
+    :class:`~matplotlib.axes.Axes`
+    '''
+
+    if ax is not None:
+      fig=ax.get_figure()
+
+    else:
+      fig,ax=plt.subplots(1,1,figsize=self._sfigs(**kwargs))
+
+    return fig,ax
 
   def _closefig(self,fig):
     '''
@@ -152,7 +168,7 @@ class PlotModels(object):
 
     '''
 
-    #trans=mpl.rcParams['savefig.transparent']
+    # trans=mpl.rcParams['savefig.transparent']
     if self.pdf is not None:
       self.pdf.savefig(figure=fig,transparent=False)
       plt.close(fig)
@@ -160,7 +176,6 @@ class PlotModels(object):
     else:
       return fig
 
-
   def plot_lines(self,models,lineIdents,useLineEstimate=True,jansky=False,
                  showBoxes=True,lineObs=None,lineObsLabel="Obs.",peakFlux=False,
                  showCont=False,xLabelGHz=False,showGrid=True,**kwargs):
@@ -331,7 +346,7 @@ class PlotModels(object):
         ylinesObsErr[:,i]=lineObs[lineIdx].flux_err
         ylinesObsErr2[0,i]=(lineObs[lineIdx].flux)/2.0
         ylinesObsErr2[1,i]=lineObs[lineIdx].flux
-        #print linesObs[i].flag
+        # print linesObs[i].flag
         if lineObs[lineIdx].flag=="ul":
           ylinesObsUl.append(ylinesObs[i]/0.5)
           ylinesObsErr[0,i]=ylinesObs[i]*0.4
@@ -370,10 +385,10 @@ class PlotModels(object):
 
     if showGrid:
       xgrid=np.array(x)
-      #ygrid=ax.get_yticks()
-      #print(ygrid)
+      # ygrid=ax.get_yticks()
+      # print(ygrid)
       ax.vlines(xgrid-0.5,ymin=ax.get_ylim()[0],ymax=ax.get_ylim()[1],linestyle="solid",linewidth=0.5,color="grey",zorder=-100)
-      #ax.hlines(ygrid,xmin=ax.get_xlim()[0],xmax=ax.get_xlim()[1],linestyle="solid",linewidth=0.5,color="grey",zorder=100)
+      # ax.hlines(ygrid,xmin=ax.get_xlim()[0],xmax=ax.get_xlim()[1],linestyle="solid",linewidth=0.5,color="grey",zorder=100)
       ax.yaxis.grid(color="grey",zorder=-100)
 
     ax.set_xticklabels(lticks,rotation='70',minor=False)
@@ -386,7 +401,6 @@ class PlotModels(object):
 
     return self._closefig(fig)
 
-
   def plot_NH(self,models,sdscale=False,marker=None,**kwargs):
     '''
     Plots the total vertical hydrogen column density as a function of radius for
@@ -438,7 +452,7 @@ class PlotModels(object):
     if sdscale:
       ax2=ax.twinx()
       ax2.set_ylabel(r"$\Sigma\,\mathrm{[g\,cm^{-2}]}$")
-      #ax2.set_xlim(xmin, xmax)
+      # ax2.set_xlim(xmin, xmax)
 
         # this needs to be done to get the correct scale
       ylim=np.array(ax.get_ylim())*model.muH
@@ -455,8 +469,7 @@ class PlotModels(object):
 
     return self._closefig(fig)
 
-
-  def plot_tcdspec(self,models,species,relToH=False,facGrayBox=3.0,**kwargs):
+  def plot_tcdspec(self,models,species,relToH=False,facGrayBox=3.0,ice=False,ax=None,**kwargs):
     '''
     Plots the total vertical columndensity for the given species for all the models
     as a function of the radius.
@@ -480,9 +493,13 @@ class PlotModels(object):
       Set to `None` if it should not be plotted.
       DEFAULT: 3
 
+    ice : boolean
+      Plot also the ice phase column density if the ice species exists.
+
     '''
     print("PLOT: plot_tcdspec ...")
-    fig,ax=plt.subplots(1,1,figsize=self._sfigs(**kwargs))
+    fig,ax=self._initfig(ax,**kwargs)
+    # fig,ax=plt.subplots(1,1,figsize=self._sfigs(**kwargs))
 
     xmin=1.e100
     xmax=0
@@ -507,6 +524,18 @@ class PlotModels(object):
         if line.get_linestyle()=="--":
           self._set_dashes(line)
 
+        if ice and species+"#" in model.spnames:
+          y=model.cdnmol[:,0,model.spnames[species+"#"]]
+          if relToH==True:
+            y=y/model.NHver[:,0]
+
+          lineice,=ax.plot(x,y,"--",marker=None,linewidth=linewidth,
+                  color=line.get_color())
+          # FIXME: should only be applied for real dashed
+          # However, there are other ways to set the dashed line style (in words ..)
+          if lineice.get_linestyle()=="--":
+            self._set_dashes(line)
+
         iplot=iplot+1
 
         if min(x)<xmin: xmin=min(x)
@@ -520,24 +549,27 @@ class PlotModels(object):
     # TODO: make a paremter so that it is possible to provide the index
     # of the reference model
     if facGrayBox is not None:
-      #x = models[-2].x[:, 0]
-      #y = models[-2].cdnmol[:, 0, model.spnames[species]]
+      # x = models[-2].x[:, 0]
+      # y = models[-2].cdnmol[:, 0, model.spnames[species]]
       ax.fill_between(x,y/facGrayBox,y*facGrayBox,color='0.8')
     ax.set_xlim(xmin,xmax)
     ax.semilogy()
 
     ax.set_xlabel(r"r [au]")
+    speclabel=prodimopy.plot.spnToLatex(species)
+    if ice and species+"#" in model.spnames:
+      speclabel=speclabel+","+prodimopy.plot.spnToLatex(species+"#")
+
     if relToH==True:
-      ax.set_ylabel(r"average $\mathrm{\epsilon("+prodimopy.plot.spnToLatex(species)+")}$ ")
+      ax.set_ylabel(r"average $\mathrm{\epsilon("+speclabel+")}$ ")
     else:
-      ax.set_ylabel(r"N$_\mathrm{"+prodimopy.plot.spnToLatex(species)+"}$ "+"[cm$^{-2}$]")
+      ax.set_ylabel(r"N$_\mathrm{"+speclabel+"}$ "+"[cm$^{-2}$]")
 
     self._dokwargs(ax,**kwargs)
     self._legend(ax,**kwargs)
 
     return self._closefig(fig)
 
-
   def plot_tauline(self,models,lineIdent,xlog=True,**kwargs):
     """
     Plots the line optical depths as a function of radius for a given line.
@@ -606,7 +638,6 @@ class PlotModels(object):
 
     return self._closefig(fig)
 
-
   def plot_avgabun(self,models,species,**kwargs):
     '''
     Plots the average abundance of the species as a function of radius.
@@ -654,7 +685,6 @@ class PlotModels(object):
 
     return self._closefig(fig)
 
-
   def plot_abunvert(self,models,r,species,**kwargs):
     '''
     Plot abundances at the given radius `r` as a function of height.
@@ -718,8 +748,7 @@ class PlotModels(object):
 
     return self._closefig(fig)
 
-
-  def plot_radial(self,models,fields,ylabel,zidx=0,**kwargs):
+  def plot_radial(self,models,fields,ylabel,zidx=0,ax=None,showlegend=True,**kwargs):
     '''
     Plots a quantitiy in radial direction. Fields must have the same number
     of entries as models and must contain arrays with the dimension of nx.
@@ -751,7 +780,7 @@ class PlotModels(object):
 
     '''
     print("PLOT: radial ...")
-    fig,ax=plt.subplots(1,1,figsize=self._sfigs(**kwargs))
+    fig,ax=self._initfig(ax)
 
     iplot=0
     xmin=1.e100
@@ -780,11 +809,10 @@ class PlotModels(object):
     ax.set_ylabel(ylabel)
 
     self._dokwargs(ax,**kwargs)
-    self._legend(ax,**kwargs)
+    if showlegend: self._legend(ax,**kwargs)
 
     return self._closefig(fig)
 
-
   # FIXME: plot radial and plot_midplane are more or less the same thing
   # extend plot_radial so that also a string can be used as a field name
   # e.g. that does not require to bould the fields array ...
@@ -866,8 +894,8 @@ class PlotModels(object):
 
     # Some special stuff for the XRT paper, because it did not
     # work with patches
-    #ax.axhline(1.e-17,linestyle="-",color="0.7",linewidth=1.0,zorder=-20)
-    #ax.axhline(1.e-19,linestyle="--",color="0.7",linewidth=1.0,zorder=-20)
+    # ax.axhline(1.e-17,linestyle="-",color="0.7",linewidth=1.0,zorder=-20)
+    # ax.axhline(1.e-19,linestyle="--",color="0.7",linewidth=1.0,zorder=-20)
 
     # TODO: check this patches stuff again, it seems to cause problems
     # in some of the pdf viewers
@@ -896,7 +924,6 @@ class PlotModels(object):
 
     return self._closefig(fig)
 
-
   def plot_vertical_nH(self,models,r,field,ylabel,species=None,patches=None,showR=True,**kwargs):
     '''
     Plots a quantity (field) as a function of height (column density) at a certain
@@ -927,7 +954,7 @@ class PlotModels(object):
 
         isstr=False
         # FIXME: the python 3 compiler shows an error here for basestring
-        try: # this is for pyhton 2/3 compatibility
+        try:  # this is for pyhton 2/3 compatibility
           isstr=isinstance(field,basestring)
         except NameError:
           isstr=isinstance(field,str)
@@ -962,14 +989,14 @@ class PlotModels(object):
         ax.add_patch(p)
 
     # some special stuff for the XRT paper
-    #ax.axhline(1.e-17,linestyle="-",color="0.7",linewidth=1.0,zorder=-20)
-    #ax.axhline(1.e-19,linestyle="--",color="0.7",linewidth=1.0,zorder=-20)
-    #idxr=np.argmin(np.abs(model.x[:,0]-r))
-    #idxNHrad=np.argmin(np.abs(model.NHrad[idxr,:]-2.e24))
-    #nhVer=np.log10(model.NHver[idxr,idxNHrad])
-    #ax.axvline(nhVer,color="0.7",zorder=-20,linewidth=1.0)
+    # ax.axhline(1.e-17,linestyle="-",color="0.7",linewidth=1.0,zorder=-20)
+    # ax.axhline(1.e-19,linestyle="--",color="0.7",linewidth=1.0,zorder=-20)
+    # idxr=np.argmin(np.abs(model.x[:,0]-r))
+    # idxNHrad=np.argmin(np.abs(model.NHrad[idxr,:]-2.e24))
+    # nhVer=np.log10(model.NHver[idxr,idxNHrad])
+    # ax.axvline(nhVer,color="0.7",zorder=-20,linewidth=1.0)
 
-    #ax.semilogx()
+    # ax.semilogx()
     ax.semilogy()
     if showR:
       ax.set_xlabel(r"$\mathrm{log\,N_{<H>,ver}\,[cm^{-2}]}$ at "+rstr)
@@ -983,19 +1010,47 @@ class PlotModels(object):
     self._dokwargs(ax,**kwargs)
     self._legend(ax,**kwargs)
 
-    self.pdf.savefig()
-    plt.close(fig)
+    return self._closefig(fig)
 
-
-  def plot_vertical(self,models,r,fieldname,ylabel,species=None,ylog=True,zr=True,**kwargs):
+  def plot_vertical(self,models,r,fieldname,ylabel,fieldnameIdx=None,species=None,ylog=True,xlog=False,
+                    zr=True,xfield="zr",showlegend=True,ax=None,**kwargs):
     '''
-    Plots a quantity (fieldname) as a function of height (z/r) at a certain
-    radius.
+    Plots a quantity (fieldname) as a function of height (various options) at
+    the given radius.
+
+    The routine searches for the closest radial grid point for the given radius,
+    no interpolation is done.
+
+    Parameters
+    ----------
+
+    fieldname : str
+      The name of a field(attribute) of the model data object (:class:`~prodimopy.read.Data_ProDiMo`)
+
+    fieldnameIdx : int
+      An additional index in case the quantity to plot has more than the two spatial dimensions.
+      e.g. fiels[ix,iz,fieldnameIdx]
+
+    species : str
+      A species name. If provided the abundance of this species is plotted.
+      `fieldname` has to be `nmol` in that case.
+
+    xfield : str
+      Chose the field/quantity that should be used for the x-axis (height).
+      Options:
+
+      * `zr` z/r, default
+      * `NHver` vertial hydrogen column density.
+      * `AVver` vertial visual extinction.
+      * `tg` gas temperature.
+
+      If none of those or an unknown value is given the `z` coordinate is used.
+
     '''
     print("PLOT: plot_vertical ...")
     rstr=r"r$\approx${:.1f} au".format(r)
 
-    fig,ax=plt.subplots(1,1)
+    fig,ax=self._initfig(ax,**kwargs)
 
     iplot=0
     xmin=1.e100
@@ -1006,16 +1061,34 @@ class PlotModels(object):
       # closest radial point to given radius
       ix=(np.abs(model.x[:,0]-r)).argmin()
 
-      if zr:
+      if zr and xfield=="zr":
         x=model.z[ix,:]/model.x[ix,0]
+      #  xfield=="nH" is just for backward compatibility, should not be use
+      elif xfield=="NHver" or xfield=="nH":
+        old_settings=np.seterr(divide='ignore')
+        x=np.log10(model.NHver[ix,:])
+        np.seterr(**old_settings)  # reset to defaul
+        zr=False
+      elif xfield=="tg":
+        x=model.tg[ix,:]
+        zr=False
+      elif xfield=="AVver":
+        old_settings=np.seterr(divide='ignore')
+        x=np.log10(model.AVver[ix,:])
+        np.seterr(**old_settings)
+        zr=False
       else:
         x=model.z[ix,:]
 
       if species==None:
-        y=getattr(model,fieldname)[ix,:]
+        getattr(model,fieldname)
+        if fieldnameIdx is not None:
+          y=getattr(model,fieldname)[ix,:,fieldnameIdx]
+        else:
+          y=getattr(model,fieldname)[ix,:]
       else:
         y=getattr(model,fieldname)[ix,:,model.spnames[species]]
-        y=y/model.nHtot[ix,:] # abundance"
+        y=y/model.nHtot[ix,:]  # abundance"
 
       ax.plot(x,y,self.styles[iplot],marker=None,color=self.colors[iplot],label=model.name)
 
@@ -1034,28 +1107,34 @@ class PlotModels(object):
 
     if "ylim" in kwargs:
       ax.set_ylim(kwargs["ylim"])
-    #else:
+    # else:
     #  ax.set_ylim(ymin,ymax)
 
     if ylog: ax.semilogy()
+    if xlog: ax.semilogx()
 
     if zr:
       ax.invert_xaxis()
       ax.set_xlabel(r"z/r @ "+rstr)
+    elif xfield=="NHver" or xfield=="nH":
+      ax.set_xlabel(r"$\mathrm{\log\,N_{<H>}\,[cm^{-2}]}$ @"+rstr)
+    elif xfield=="AVver":
+      ax.set_xlabel(r"$\mathrm{\log\,A_{V,ver}}$ @"+rstr)
+    elif xfield=="tg":
+      ax.set_xlabel(r"$\mathrm{\log\,T_{gas}\,[K]}$ @"+rstr)
+      ax.invert_xaxis()
     else:
       ax.set_xlabel(r"z [au] @ "+rstr)
-      ax.semilogx()
       ax.invert_xaxis()
 
     ax.set_ylabel(ylabel)
 
     # FIXME: make it general
-    #self._dokwargs(ax,kwargs)
-    self._legend(ax,**kwargs)
+    self._dokwargs(ax,**kwargs)
+    if showlegend: self._legend(ax,**kwargs)
 
     return self._closefig(fig)
 
-
   def plot_abunradial(self,models,species,perH2=False,**kwargs):
     '''
     Plots the abundance of the given species as a function of radius for the
@@ -1088,11 +1167,12 @@ class PlotModels(object):
       else:
         y=model.nmol[:,0,model.spnames[species]]/model.nHtot[:,0]
 
-      if iplot==0 or iplot==(len(models)-1):
-        line,=ax.plot(x,y,self.styles[iplot],marker=None,color=self.colors[iplot],label=model.name,linewidth=2.5)
-      else:
-        line,=ax.plot(x,y,self.styles[iplot],marker=None,color=self.colors[iplot],label=model.name)
-        if line.is_dashed(): self._set_dashes(line)
+      # this is not general, removed it
+      # if iplot==0 or iplot==(len(models)-1):
+      #  line,=ax.plot(x,y,self.styles[iplot],marker=None,color=self.colors[iplot],label=model.name,linewidth=2.5)
+      # else:
+      line,=ax.plot(x,y,self.styles[iplot],marker=None,color=self.colors[iplot],label=model.name)
+      if line.is_dashed(): self._set_dashes(line)
 
       iplot=iplot+1
 
@@ -1114,7 +1194,6 @@ class PlotModels(object):
 
     return self._closefig(fig)
 
-
   def plot_sed(self,models,plot_starSpec=True,sedObs=None,sedObsModels=False,unit="erg",
                reddening=False,**kwargs):
     '''
@@ -1226,8 +1305,8 @@ class PlotModels(object):
                       yerr=ysedObsErr[okidx],markeredgecolor="0.3",
                     linestyle="",fmt="o",color=sedcolor,ms=2,linewidth=1.0,
                     markeredgewidth=0.5,zorder=1000,ecolor='0.3')
-          #nokidx=np.where(plSedObs.flag!="ok")
-          #ax.plot(xsedObs[nokidx],ysedObs[nokidx],linestyle="",
+          # nokidx=np.where(plSedObs.flag!="ok")
+          # ax.plot(xsedObs[nokidx],ysedObs[nokidx],linestyle="",
           #        marker=".",markeredgecolor="0.3",markeredgewidth=0.5,
           #        color=sedcolor)
           ulidx=np.where(plSedObs.flag=="ul")
@@ -1236,7 +1315,7 @@ class PlotModels(object):
           #            yerr=ysedObsErr[okidx],markeredgecolor="0.3",markeredgewidth=0.2,
           #          linestyle="",fmt="o",color=sedcolor,ms=1,linewidth=1.0,zorder=1000,ecolor='0.3')
 
-          #ax.errorbar(xsedObs[ulidx],ysedObs[ulidx],uplims=True,
+          # ax.errorbar(xsedObs[ulidx],ysedObs[ulidx],uplims=True,
           #            yerr=ysedObsErr[okidx],markeredgecolor="0.3",markeredgewidth=0.2,
           #          linestyle="",fmt="o",color=sedcolor,ms=1,linewidth=1.0,zorder=1000,ecolor='0.3')
 
@@ -1248,7 +1327,7 @@ class PlotModels(object):
 
               ax.fill_between(spec[:,0],nu*(fnuerg-fnuergErr),nu*(fnuerg+fnuergErr),color='0.8')
 
-              #ax.errorbar(,nu*fnuerg,
+              # ax.errorbar(,nu*fnuerg,
               #            yerr=nu*fnuergErr,ms=2,linewidth=0.4,
               #            markeredgewidth=0.5,zorder=1000,ecolor="0.5",
               #            linestyle="-",color="0.3")
@@ -1278,20 +1357,23 @@ class PlotModels(object):
 
     self._dokwargs(ax,**kwargs)
 
-    self._legend(ax)
+    self._legend(ax,**kwargs)
 
     return self._closefig(fig)
 
-
   def plot_dust_opac(self,models,xenergy=False,opactype="both",**kwargs):
     '''
-    Plots the dust opacities, absorption and scattering
+    Plots the dust opacities, absorption, scattering or extinction.
     The pseudo anisotropic scattering is not plotted yet.
 
-    opactype: both absorption and scattering
-              abs  only absorption
-              sca  only scattering
-              pasca only anisotropic scattering
+    Parameters
+    ----------
+    opactype : str
+      `both` absorption and scattering (separately),
+      `abs`  only absorption,
+      `sca`  only scattering,
+      `ext`  only extinction
+
     '''
     print("PLOT: plot_dust_opac ...")
     fig,ax=plt.subplots(1,1,figsize=self._sfigs(**kwargs))
@@ -1308,41 +1390,52 @@ class PlotModels(object):
       else:
         x=model.dust.lam
 
-      if opactype=="both" or opactype=="abs":
-        y=model.dust.kabs
+      if opactype=="ext":
+        y=model.dust.kabs+model.dust.ksca
         # TODO: this is not consistent with providing the linestyle via the
         # command line
 
-        if opactype=="both":
-          label=model.name+" abs"
-          linest="-"
-        else:
-          label=model.name
-          linest=self.styles[iplot]
+        label=model.name
+        linest=self.styles[iplot]
 
-        lineabs,=ax.plot(x,y,color=self.colors[iplot],linestyle=linest,
-              label=label)
+        ax.plot(x,y,color=self.colors[iplot],linestyle=linest,label=label)
 
-      if opactype=="both" or opactype=="sca":
-        # TODO: this is not consistent with providing the linestyle via the
-        # command line
-        y=model.dust.ksca
+      else:
+        if opactype=="both" or opactype=="abs":
+          y=model.dust.kabs
+          # TODO: this is not consistent with providing the linestyle via the
+          # command line
 
-        if opactype=="both":
-          col=lineabs.get_color()
-          linest="--"
-          label=model.name+" sca"
-        else:
-          col=self.colors[iplot]
-          linest=self.styles[iplot]
-          label=model.name
+          if opactype=="both":
+            label=model.name+" abs"
+            linest="-"
+          else:
+            label=model.name
+            linest=self.styles[iplot]
 
-        line,=ax.plot(x,y,color=col,linestyle=linest,
-              label=label)
-        if line.is_dashed(): self._set_dashes(line)
+          lineabs,=ax.plot(x,y,color=self.colors[iplot],linestyle=linest,
+                label=label)
 
-      #y=model.dust.ksca_an*1000.0
-      #line, =ax.plot(x, y, color=lineabs.get_color(),linestyle=":",
+        if opactype=="both" or opactype=="sca":
+          # TODO: this is not consistent with providing the linestyle via the
+          # command line
+          y=model.dust.ksca
+
+          if opactype=="both":
+            col=lineabs.get_color()
+            linest="--"
+            label=model.name+" sca"
+          else:
+            col=self.colors[iplot]
+            linest=self.styles[iplot]
+            label=model.name
+
+          line,=ax.plot(x,y,color=col,linestyle=linest,
+                label=label)
+          if line.is_dashed(): self._set_dashes(line)
+
+      # y=model.dust.ksca_an*1000.0
+      # line, =ax.plot(x, y, color=lineabs.get_color(),linestyle=":",
       #        label=model.name+" sca (pa)")
 
       iplot=iplot+1
@@ -1355,10 +1448,12 @@ class PlotModels(object):
     # TODO: sometimes it is just zero
     if ymin<1.e-100: ymin=1.e-20
     # set defaults, can be overwritten by the kwargs
-    #ax.set_xlim(xmin,xmax)
-    #ax.set_ylim([ymin,ymax])
+    # ax.set_xlim(xmin,xmax)
+    # ax.set_ylim([ymin,ymax])
     ax.semilogx()
     ax.semilogy()
+    if opactype=="ext":
+      ax.set_ylabel(r"ext. opacity $\mathrm{[cm^2 g(dust)^{-1}]}$")
     if opactype=="both":
       ax.set_ylabel(r"opacity $\mathrm{[cm^2 g(dust)^{-1}]}$")
     elif opactype=="abs":
@@ -1376,13 +1471,12 @@ class PlotModels(object):
 
     return self._closefig(fig)
 
-
-  def plot_starspec_xray(self,models,nuInu=False,**kwargs):
+  def plot_starspec_xray(self,models,nuInu=False,ax=None,**kwargs):
     '''
     Plots the X-ray spectrum.
     '''
     print("PLOT: plot_starspec_xray ...")
-    fig,ax=plt.subplots(1,1,figsize=self._sfigs(**kwargs))
+    fig,ax=self._initfig(ax,**kwargs)
 
     iplot=0
     xmax=0
@@ -1415,7 +1509,7 @@ class PlotModels(object):
 
       if max(y)>ymax: ymax=max(y)
 
-    xmin=0.1 # keV
+    xmin=0.1  # keV
 
     # TODO: sometimes it is just zero
     if ymin<1.e-100: ymin=1.e-20
@@ -1434,11 +1528,9 @@ class PlotModels(object):
 
     self._legend(ax,**kwargs)
 
-    self.pdf.savefig()
-    plt.close(fig)
+    return self._closefig(fig)
 
-
-  def plot_starspec(self,models,**kwargs):
+  def plot_starspec(self,models,step=10,xunit="micron",nuInu=True,ax=None,**kwargs):
     '''
     Plots the full Stellar Spectrum for each model.
 
@@ -1447,9 +1539,18 @@ class PlotModels(object):
     models : list(:class:`~prodimopy.read.Data_ProDiMo`)
       the models to plot
 
+    step : int
+      Only plot every `step` point of the spectrum.
+
+    xunit : str
+      Unit for the x-axis. Currently `mircon` and `eV` are possible.
+
+    nuInu : boolean
+      Plut nu times Inu instead of Inu only.
+
     '''
     print("PLOT: plot_starspec ...")
-    fig,ax=plt.subplots(1,1)
+    fig,ax=self._initfig(ax,**kwargs)
 
     iplot=0
     xmin=1.e100
@@ -1458,11 +1559,24 @@ class PlotModels(object):
     ymax=-1.e00
     for model in models:
 
-      x=model.starSpec.lam[0::10]
-      y=(model.starSpec.nu*model.starSpec.Inu)[0::10]
+      x=model.starSpec.lam[0::step]
+      if xunit=="eV":
+        x=(x*u.micron).to(u.eV,equivalencies=u.spectral()).value
+        # switch the axes
+        xmin=(1000.0*u.micron).to(u.eV,equivalencies=u.spectral()).value
+        xmax=x.max()
+        xlabel=r"energy [eV]"
+      else:
+        xmin=x.min()
+        xmax=1000.0
+        xlabel=r"wavelength [$\mathrm{\mu}$m]"
+
+      y=(model.starSpec.Inu)[0::step]
+      if nuInu:
+        y=y*model.starSpec.nu[0::step]
 
       ax.plot(x,y,color=self.colors[iplot],linestyle=self.styles[iplot],
-              ms=2,markeredgecolor=self.colors[iplot],
+              ms=2,markeredgecolor=self.colors[iplot],linewidth=1.0,
               label=model.name)
       # marker=self.markers[iplot],
 
@@ -1481,8 +1595,12 @@ class PlotModels(object):
     ax.set_ylim([ymin,ymax])
     ax.semilogx()
     ax.semilogy()
-    ax.set_ylabel(r"$\mathrm{\nu F_{\nu}\,[erg\,cm^{-2}\,s^{-1}]}$")
-    ax.set_xlabel(r"wavelength [$\mathrm{\mu}$m]")
+    if nuInu:
+      ax.set_ylabel(r"$\mathrm{\nu I_\nu\,[erg\,cm^{-2}\,s^{-1}\,sr^{-1}]}$")
+    else:
+      ax.set_ylabel(r"$\mathrm{I_\nu\,[erg\,cm^{-2}\,s^{-1}\,sr^{-1}\,Hz^{-1}]}$")
+#    ax.set_ylabel(r"$\mathrm{\nu F_{\nu}\,[erg\,cm^{-2}\,s^{-1}]}$")
+    ax.set_xlabel(xlabel)
 
     self._dokwargs(ax,**kwargs)
 
@@ -1490,14 +1608,54 @@ class PlotModels(object):
 
     return self._closefig(fig)
 
-
   def plot_line_profil(self,models,wl,ident=None,linetxt=None,lineObs=None,
-                       normalized=False,**kwargs):
+                       unit="Jy",normalized=False,**kwargs):
+
+    print("WARN: plot_line profil ... please use plot_lineprofile instead. This routine will be removed in the next version.")
+    return self.plot_lineprofile(models,wl,ident=ident,linetxt=linetxt,lineObs=lineObs,
+                       unit=unit,normalized=normalized,**kwargs)
+
+  def plot_lineprofile(self,models,wl,ident=None,contsub=True,linetxt=None,lineObs=None,
+                       normalized=False,convolved=False,style=None,ax=None,**kwargs):
     '''
     Plots the line profile for the given line (id wavelength and line ident)
+
+    Parameters
+    ----------
+    models : array_like(:class:`~prodimopy.read.Data_ProDiMo`,dim=1)
+      the models
+
+    wl : float
+      The wavelength of the line in micrometer. Plotted is the line with the wavelength
+      closest to `wl`.
+
+    ident : str
+      The optional line ident which is additionally use to identify the line.
+
+    contsub : boolean
+      Remove the continuum. Default: `True`
+
+    linetxt : str
+      A string the is used as the label for the line. Default: `None`
+
+    lineObs : array_like(ndim=1)
+      list of :class:`prodimopy.read.DataLineObs` objects. Must be consistent with the list of
+      lines from the line radiative transfer.
+
+    normalized : boolean
+      if `True` normalize the profile to the peak flux of each line
+
+    convolved : boolean
+      if `True` plot the convolved profile.
+
+    style : str
+      if style is `step` the profile is plotted as a step function assuming
+      the values are the mid point of the bin.
+
     '''
-    print("PLOT: plot_line_profile ...")
-    fig,ax=plt.subplots(1,1,figsize=self._sfigs(**kwargs))
+
+    print("PLOT: plot_lineprofile ...")
+    fig,ax=self._initfig(ax,**kwargs)
 
     iplot=0
     xmin=1.e100
@@ -1512,14 +1670,27 @@ class PlotModels(object):
       if iplot==0 and linetxt is None:
         linetxt=line.species+"@"+"{:.2f}".format(line.wl)+" $\mathrm{\mu m}$"
       x=line.profile.velo
-      y=line.profile.flux-line.fcont  # remove the continuum
+
+      # Remove the continuum
+
+      if convolved:
+        y=line.profile.flux_conv
+        if contsub:
+          y=y-line.profile.flux_conv[0]
+      else:
+        y=line.profile.flux
+        if contsub:
+          y=y-line.profile.flux[0]
 
       # normalize to their peak
       if normalized:
-        print("Normalize it")
         y=y/np.max(y)
 
-      ax.plot(x,y,self.styles[iplot],marker=None,color=self.colors[iplot],label=model.name)
+      if style=="step":
+        ax.step(x,y,self.styles[iplot],marker=None,color=self.colors[iplot],label=model.name,where="mid")
+      else:
+        ax.plot(x,y,self.styles[iplot],marker=None,color=self.colors[iplot],label=model.name)
+        # ax.plot(x,line.profile.flux_dust,":",marker=None,color=self.colors[iplot],label=model.name)
 
       iplot=iplot+1
 
@@ -1542,8 +1713,18 @@ class PlotModels(object):
       line=lineObs[lineIdx]
       if line.profile is not None:
         x=line.profile.velo
-        y=line.profile.flux  # remove the continuum
+        y=line.profile.flux
+        # # FIXME:
+        # if line.profile.flux_unit=="ErgAng":
+          # y=line.profile.fluxErgAng
+        # else:
+          # y=line.profile.flux
+
+        # normalize to their peak
+        if normalized:
+          y=y/np.max(y)
         if line.profileErr is not None:
+          # FIXME: profile error also needs to be converted to the proper units
           ax.fill_between(x,y-line.profileErr ,y+line.profileErr,color='0.8',zorder=0)
         ax.plot(x,y,marker=None,color="black",label="Obs.",zorder=0)
 
@@ -1557,7 +1738,14 @@ class PlotModels(object):
     ax.set_ylim([ymin,ymax*1.1])
 
     ax.set_xlabel(r"$\mathrm{velocity\,[km\;s^{-1}}$]")
-    ax.set_ylabel(r"$\mathrm{flux\,[Jy]}$")
+
+    if normalized:
+      ax.set_ylabel("normalized flux")
+    else:
+      if line.profile.flux_unit=="ErgAng":
+        ax.set_ylabel(r"$\mathrm{flux\,[erg s^{-1}cm^{-2}\AA^{-1}]}$")
+      else:
+        ax.set_ylabel(r"$\mathrm{flux\,[Jy]}$")
 
     self._dokwargs(ax,**kwargs)
 
